require File.expand_path("../helper.rb", __FILE__)

class SimpleProcessor
  attr_reader :foo, :bar

  def process(line_no, cmd, *args)
    case cmd
    when "foo"
      @foo = args
    when "bar"
      @bar = args
    else
      return false
    end

    true
  end
end

SIMPLE_FILE = <<END
foo 1
bar 1 2
foo 1 2 3
END

SIMPLE_FILE_WITH_COMMENTS = <<END
foo 1
# this is a comment
bar 1 2

# and another one
# and another...
foo 1 2 3
END

class GroupProcessor
  attr_reader :foos

  def process(line_no, cmd, *args)
    case cmd
    when "foo"
      @foos ||= []
      @foos << args.first
    else
      return false
    end

    true
  end
end

class GroupedProcessor
  attr_reader :group, :bar

  def process(line_no, cmd, *args)
    case cmd
    when "bar"
      @bar = args
    when "group"
      @group = GroupProcessor.new
      return @group
    else
      return false
    end

    true
  end
end

class InvalidGroupedProcessor
  attr_reader :group, :bar

  def process(line_no, cmd, *args)
    case cmd
    when "bar"
      @bar = args
    when "group"
      return Object.new
    else
      return false
    end

    true
  end
end

ONE_GROUP_FILE = <<END
bar 1 2

group
foo 3
foo 4
END

class RepeatableGroupedProcessor
  attr_reader :group, :bar

  def process(line_no, cmd, *args)
    case cmd
    when "bar"
      @bar = args
    when "group"
      @group ||= []
      @group << (g = GroupProcessor.new)
      return g
    else
      return false
    end

    true
  end
end

REPEATED_GROUP_FILE = <<END
bar 0

group
foo 3
foo 4

bar 1 2

group
foo 5
foo 6
END

class InvalidResponseProcessor
  def process(line_no, cmd, *args)
    return "foo"
  end
end

UNSUPPORTED_OPTION_FILE = <<END
foo 1 2
baz 1
END

class ArityCheckProcessor
  ARITY = {
    'foo' => 1,
    'baz' => true
  }

  attr_reader :foo, :bar, :baz

  def process(line_no, cmd, *args)
    case cmd
    when "foo"
      @foo = args
    when "bar"
      @bar = args
    when "baz"
      @baz = args
    else
      return false
    end

    true
  end
end

WRONG_ARITY_FILE_1 = <<END
foo 1 2
bar 1 2
baz 1 2 3 4
END

WRONG_ARITY_FILE_2 = <<END
foo 1
bar 1 2
baz
END

CORRECT_ARITY_FILE = <<END
foo 1
bar 1 2 3 4
baz 1 2 3 4
END


class TestParser < Minitest::Test
  def test_simple_processing
    b = Booby.prepare(SIMPLE_FILE)
    parsed = b.parse_with(SimpleProcessor)
    assert_equal %w{1 2 3}, parsed.foo
    assert_equal %w{1 2}, parsed.bar
  end

  def test_simple_processing_with_comments
    b = Booby.prepare(SIMPLE_FILE_WITH_COMMENTS)
    parsed = b.parse_with(SimpleProcessor)
    assert_equal %w{1 2 3}, parsed.foo
    assert_equal %w{1 2}, parsed.bar
  end

  def test_basic_groups_processing
    b = Booby.prepare(ONE_GROUP_FILE)
    parsed = b.parse_with(GroupedProcessor)
    assert_equal %w{1 2}, parsed.bar
    assert_equal %w{3 4}, parsed.group.foos
  end

  def test_multiple_groups
    b = Booby.prepare(REPEATED_GROUP_FILE)
    parsed = b.parse_with(RepeatableGroupedProcessor)
    assert_equal %w{1 2}, parsed.bar
    assert_equal %w{3 4}, parsed.group[0].foos
    assert_equal %w{5 6}, parsed.group[1].foos
  end

  def test_invalid_response
    b = Booby.prepare(ONE_GROUP_FILE)
    assert_raises Booby::InvalidResponseError do
      b.parse_with(InvalidResponseProcessor)
    end
  end

  def test_unsupported_option
    b = Booby.prepare(UNSUPPORTED_OPTION_FILE)
    assert_raises Booby::UnsupportedOptionError do
      b.parse_with(SimpleProcessor)
    end
  end

  def test_strict_arity_checking_fail
    b = Booby.prepare(WRONG_ARITY_FILE_1)
    assert_raises Booby::InvalidNumberOfParametersError do
      parsed = b.parse_with(ArityCheckProcessor)
    end
  end

  def test_minimum_arity_checking_fail
    b = Booby.prepare(WRONG_ARITY_FILE_2)
    assert_raises Booby::InvalidNumberOfParametersError do
      parsed = b.parse_with(ArityCheckProcessor)
    end
  end

  def test_arity_checking_success
    b = Booby.prepare(CORRECT_ARITY_FILE)
    parsed = b.parse_with(ArityCheckProcessor)
    assert_equal %w{1}, parsed.foo
    assert_equal %w{1 2 3 4}, parsed.bar
    assert_equal %w{1 2 3 4}, parsed.bar
  end

  def test_file_opening
    testfile = File.join($test_tmp_dir, "test.conf")
    File.open(testfile, "w+") { |f| f.puts(SIMPLE_FILE) }
    b = Booby.open(testfile)
    parsed = b.parse_with(SimpleProcessor)
    assert_equal %w{1 2 3}, parsed.foo
    assert_equal %w{1 2}, parsed.bar
  end
end
